use std::fmt::Display;

use async_trait::async_trait;
use inquire::Text;

use crate::{domain::config::{Config, Instance}, flows::{gamenight_menu::GamenightMenu, login::Login}};

use super::{Flow, FlowOutcome, FlowResult, GamenightState};

#[derive(Clone)]
pub struct Connect {
    instance: Option<Instance>
}

impl Connect {
    pub fn to(instance: Instance) -> Self {
        Self {
            instance: Some(instance)
        }
    }

    pub fn new() -> Self {
        Self {
            instance: None
        }
    }
}

#[async_trait]
impl<'a> Flow<'a> for Connect {
    async fn run(&self, state: &'a mut GamenightState) -> FlowResult<'a> {
        let mut instance = if let Some(instance) = self.instance.clone() {
            instance
        } else {
            let mut name: String;
            loop {
                name = Text::new("Name this instance: ").prompt()?;
                if state.gamenight_configuration.instances.iter().find(|x| x.name == name).is_none() {
                    break;
                }
                clear_screen::clear();
                println!("Name already in use, please provide a unique name");
            }
            let url = Text::new("What is the server URL: ").prompt()?;
            Instance::new(name, url)
        };

        let instance_name = instance.name.clone();

        state.api_configuration.base_path = instance.url.clone();
        if let Some(token) = instance.token {
            state.api_configuration.bearer_access_token = Some(token);
            let result = gamenight_api_client_rs::apis::default_api::post_token(&state.api_configuration).await;
            if let Ok(token) = result {
                let instance = state.gamenight_configuration.instances.iter_mut().find(|x| x.name == instance_name).unwrap();
                instance.token = token.jwt_token.clone();
                state.api_configuration.bearer_access_token = token.jwt_token.clone();
                Config::save(&state.gamenight_configuration)?;
                let gamenight_menu_flow = GamenightMenu::new();
                return gamenight_menu_flow.run(state).await
            }
        }

        let login_flow = Login::new();
        let (outcome, state) = login_flow.run(state).await?;

        if outcome == FlowOutcome::Successful {
            if self.instance.is_none() {
                instance.token = Some(state.api_configuration.bearer_access_token.clone().unwrap());
                state.gamenight_configuration.instances.push(instance.clone());
            } 
            else {
                let instance = state.gamenight_configuration.instances.iter_mut().find(|x| x.name == instance_name).unwrap();
                instance.token = Some(state.api_configuration.bearer_access_token.clone().unwrap());
            }

            state.gamenight_configuration.last_instance = Some(instance_name);
            
            Config::save(&state.gamenight_configuration)?;

            let gamenight_menu_flow = GamenightMenu::new();
            gamenight_menu_flow.run(state).await
        }
        else {
            Ok((outcome, state))
        }
    }
}

impl Display for Connect {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        if let Some(instance) = &self.instance {
            write!(f, "Connect to: {}", instance.name)
        } else {
            write!(f, "Connect")
        }
        
    }
}